=head1 NAME

WebService::Yahoo::SpellingSuggestion - Perl wrapper for the Yahoo! Spelling Suggestion WebService

=head1 SYNOPSIS

  use WebService::Yahoo::SpellingSuggestion;

  my $yss = WebService::Yahoo::SpellingSuggestion->new(
              appid => 'your_app_id', context => $source_text
            );

  my $suggestion = $yss->get_suggestion;

=head1 DESCRIPTION

WebService::Yahoo::SpellingSuggestion provides a simple object-oriented
wrapper around the Yahoo! Spelling Suggestion WebService.

The Yahoo! Spelling Suggestion WebService attempts to correct any mistakes
in the text submitted, returning a corrected version of the source text on
success.

=head1 EXAMPLES

    use WebService::Yahoo::SpellingSuggestion;

    my $source_text = 'brken parargaraph';

    my $yss = WebService::Yahoo::SpellingSuggestion->new(
                appid => 'your_app_id', context => $source_text
              );

    my $suggestion = $yss->get_suggestion;

    if ($suggestion) {
      print "$suggestion\n"
    } else {
      print "Either the word's correct or Yahoo doesn't have a correction\n";
    }

If you are making multiple calls, each with different source text, and
you have already initialised a WebService::Yahoo::SpellingSuggestion
object, you can reuse the object and call C<get_suggestion> with the
text to check.

    my $yss = WebService::Yahoo::SpellingSuggestion->new(
                appid => 'your_app_id', context => $source_text
              );

    my $suggestion = $yss->get_suggestion;

    ... do stuff and then later ...

    $suggestion = $yss->get_suggestion($new_source_text);

=head1 NOTES

While this module does what the documentation claims (barring any bugs)
I'd feel remiss if I didn't add a word of caution. The Yahoo
Spelling Suggestion service isn't actually very good. From my experiements
it doesn't seem to know about place names (which was important to me when I
wrote it), it seems to have problems with more than one-three words at a
time and, unless I misunderstand the service, it's not possible to determine
if a word is correct or just can't be dealt with.

=cut

###########################################################

package WebService::Yahoo::SpellingSuggestion;
use strict;
use warnings;
use LWP::UserAgent;
use URI::Escape;
use vars qw($VERSION);

$VERSION = "0.01";

=head1 FUNCTIONS

As an object-oriented module WebService::Yahoo::SpellingSuggestion
exports no functions.

=head1 CONSTRUCTOR

=over 4

=item new ( appid => 'your_app_id', context => 'your_source_text' )

This is the constructor for a new WebService::Yahoo::SpellingSuggestion
object. The C<appid> is required for you to use the Yahoo webservice and
must be requested from them. See L<SEE ALSO> for more details.

C<context> is the source text that should be checked for mis-spellings.
Both arguments are required. Although the webservice docs don't mention
a maximum length I've had the most success with small, one, two or three
word sections. Any more than this and you'll mostly get C<undef>s back.

You can also pass in C<debug>, with a true value, if you want the URL
requested and the return code printed to C<stderr>.

=back

=cut

sub new {
  my ($class, %args) = @_;
  bless \%args, $class;
}

=head1 METHODS

=over 4

=item get_suggestion ( [ $context ] )

This method sends the request and returns a scalar containing the corrected
text or an C<undef> if either the words are correct OR the Yahoo service
doesn't know what to make of them; which is less useful than I'd hoped.

If invoked without an argument the C<context> provided in C<new> is
used. If an argument is passed it is assumed to be source text that
should be checked. This was added as a convenience for working with
multiple pieces of text.

C<get_suggestion> returns a scalar with the corrected text on success and
undef on failure or when the words are correct.

=back

=cut

sub get_suggestion {
  my $self = shift;
  my $url  = 'http://search.yahooapis.com/WebSearchService/V1/spellingSuggestion';

  my $source_text = shift || $self->{context};
  my $escaped_st = uri_escape($source_text);

  my $full_url = "$url?appid=" . $self->{appid} . "&query=$escaped_st";

  my $ua = LWP::UserAgent->new;
  $ua->timeout(20);

  warn "Request: '$full_url'\n" if $self->{debug};
  my $response = $ua->get( $full_url );
  warn "Response Code: ", $response->status_line, "\n" if $self->{debug};
  return undef unless $response->is_success;

  my $content = $response->content;

  $content =~ m!<Result>(.*?)</Result>!;
  return $1; # or explict undef
}

1;

#######################################################################

=head1 DEPENDENCIES

WebService::Yahoo::SpellingSuggestion requires the following modules:

L<LWP::UserAgent>
L<URI::Escape>

=head1 SEE ALSO

For information about the Yahoo! Spelling Suggestion service -
L<http://developer.yahoo.com/search/web/V1/spellingSuggestion.html>

To sign up for an application key -
L<http://api.search.yahoo.com/webservices/register_application>

=head1 LICENCE AND COPYRIGHT

Copyright (C) 2006 Dean Wilson.  All Rights Reserved.

This module is free software; you can redistribute it and/or modify it
under the same terms as Perl itself.

=head1 AUTHOR

Dean Wilson <dean.wilson@gmail.com>

=cut
